#define _MAIN_

#pragma PECDEF (0)  // Reserve space for PEC pointers, channel 0

#include <reg167.h>
#include <stdio.h>
#include <intrins.h>

#include <intmac.h>
#include <interp.h>
#include <tunemaps.h>
#include <main.h>
#include <comms.h>


/*** Initialise Compare Unit On Timer 0 ***/

void T0_init(void) {

   unsigned int i ;
   char far *tdc_regs = (char far *) TDC_Regs_Base ;

   DP2 = 0x7fff ; // Set up P2.0-P2.11 as outputs
   P2 = 0x8000 ;

   CCM0 = 0x0005 ;  // Compare mode 1, timer 0
   CCM0 |= 0x0050 ;
   CCM0 |= 0x0500 ;
   CCM0 |= 0x5000 ;

   CCM1 = 0x0005 ;
   CCM1 |= 0x0050 ;
   CCM1 |= 0x0500 ;
   CCM1 |= 0x5000 ;

   CCM2 = 0x0005 ;
   CCM2 |= 0x0050 ;
   CCM2 |= 0x0500 ;
   CCM2 |= 0x5000 ;

   Set_Priority_08(CC0IC) ;  // Set up injector interrupts
   Set_Group_Lvl_0(CC0IC) ; 
   Set_Priority_08(CC1IC) ; 
   Set_Group_Lvl_1(CC1IC) ; 
   Set_Priority_08(CC2IC) ; 
   Set_Group_Lvl_2(CC2IC) ; 
   Set_Priority_08(CC3IC) ; 
   Set_Group_Lvl_3(CC3IC) ; 

   Set_Priority_09(CC4IC) ; 
   Set_Group_Lvl_0(CC4IC) ; 
   Set_Priority_09(CC5IC) ; 
   Set_Group_Lvl_1(CC5IC) ; 
   Set_Priority_09(CC6IC) ; 
   Set_Group_Lvl_2(CC6IC) ; 
   Set_Priority_09(CC7IC) ; 
   Set_Group_Lvl_3(CC7IC) ; 

   Set_Priority_10(CC8IC) ; 
   Set_Group_Lvl_0(CC8IC) ; 
   Set_Priority_10(CC9IC) ; 
   Set_Group_Lvl_1(CC9IC) ; 
   Set_Priority_10(CC10IC) ; 
   Set_Group_Lvl_2(CC10IC) ; 
   Set_Priority_10(CC11IC) ; 
   Set_Group_Lvl_3(CC11IC) ; 
/*
   CC0IC = 0x20 ;
   CC1IC = 0x21 ;
   CC2IC = 0x22 ;
   CC3IC = 0x23 ;
   CC4IC = 0x24 ;
   CC5IC = 0x25 ;
   CC6IC = 0x26 ;
   CC7IC = 0x27 ;
   CC8IC = 0x28 ;
   CC9IC = 0x29 ;
   CC10IC = 0x2a ;
   CC11IC = 0x2d ;
*/
   Set_Priority_13(CC15IC) ;  // TDC int is highest priority 
   Set_Group_Lvl_0(CC15IC) ; 

   CC15IE = 1 ;      // Enable TDC interrupt
   CCM3 = 0x2000 ;   // Input capture on negative edge

   T01CON = 0x0002 ; // 1.6us per count
   T0IC = 0 ;  // No timer 0 overflow
   T0R = 1 ;   // Start timer 0
   
   for(i = 0 ; i < TDC_Regs_Len ; i++) {

      tdc_regs[i] = 0x00 ;  // Clear out registerbank used by TDC int
      }
   }


/*** Test Crankshaft TDC Interrupt ***/

void tdc_int(void) interrupt 0x1f using TDCREGS {

   unsigned int temp, time_for_60 ;  // Make this a register variable
   unsigned int time_last_60 ;  // These are really static but dedicated 
   unsigned int tooth_count ;   // regbank TDCREGS makes them like static 
                                // register variables

   /*** Read Pressure Channel For Synchronous Fuelling ***/

   ADCRQ  = 1 ; // Request injection read 

   time_for_60 = CC15 - time_last_60 ;  // Find time between references

   /*** Ensure injector turned off at end of 720 deg cycle ***/

   CCx_ptr[tooth_count] = CC15 ;  // Prevent further compare matches; 
                                  // timer rate must be < runtime of next line!
   P2 &= clear_port2_bits[tooth_count] ; // Make sure pin is low                           

   temp = inj_delay_angle/Max_Timeout_Angle ;
   timeout_angle[tooth_count] = MDH ;  // Decide angle to timeout over.  This 
                                       // direct access to MDH gives the 
                                       // remainder without doing a DIV again.

   temp += tooth_count ;               // Decide which tooth to time out from
   if(temp >= No_Of_Cylinders) {       // Limit to correct no of teeth per cycle
      temp -= No_Of_Cylinders;
      }

   timeout_tooth[tooth_count] = temp ;

   /*** Check If We Need To Timeout From This Tooth ***/

   if(timeout_tooth[0] == tooth_count) {

      temp = timeout_angle[0] ; 
      temp = ((unsigned long)time_for_60 * temp)/Max_Timeout_Angle ; // Inline DIV
      temp += Fix_Time ;
      CC0 = temp + T0  ;
      CC0IR = 0 ;   // Could be done with a pointer CCxIC!
      CC0IE = 1 ;
      }
   
   if(timeout_tooth[1] == tooth_count) {

      temp = timeout_angle[1] ; 
      temp = ((unsigned long)time_for_60 * temp)/Max_Timeout_Angle ; // Inline DIV
      temp += Fix_Time ;
      CC1 = temp + T0  ;
      CC1IR = 0 ;
      CC1IE = 1 ;
      }
   
   if(timeout_tooth[2] == tooth_count) {

      temp = timeout_angle[2] ; 
      temp = ((unsigned long)time_for_60 * temp)/Max_Timeout_Angle ; // Inline DIV
      temp += Fix_Time ;
      CC2 = temp + T0  ;
      CC2IR = 0 ;
      CC2IE = 1 ;
      }
   
   if(timeout_tooth[3] == tooth_count) {

      temp = timeout_angle[3] ; 
      temp = ((unsigned long)time_for_60 * temp)/Max_Timeout_Angle ; // Inline DIV
      temp += Fix_Time ;
      CC3 = temp + T0  ;
      CC3IR = 0 ;
      CC3IE = 1 ;
      }
   
   if(timeout_tooth[4] == tooth_count) {

      temp = timeout_angle[4] ; 
      temp = ((unsigned long)time_for_60 * temp)/Max_Timeout_Angle ; // Inline DIV
      temp += Fix_Time ;
      CC4 = temp + T0  ;
      CC4IR = 0 ;
      CC4IE = 1 ;
      }
   
   if(timeout_tooth[5] == tooth_count) {

      temp = timeout_angle[5] ; 
      temp = ((unsigned long)time_for_60 * temp)/Max_Timeout_Angle ; // Inline DIV
      temp += Fix_Time ;
      CC5 = temp + T0  ;
      CC5IR = 0 ;
      CC5IE = 1 ;
      }
   
   if(timeout_tooth[6] == tooth_count) {

      temp = timeout_angle[6] ; 
      temp = ((unsigned long)time_for_60 * temp)/Max_Timeout_Angle ; // Inline DIV
      temp += Fix_Time ;
      CC6 = temp + T0  ;
      CC6IR = 0 ;
      CC6IE = 1 ;
      }
   
   if(timeout_tooth[7] == tooth_count) {

      temp = timeout_angle[7] ; 
      temp = ((unsigned long)time_for_60 * temp)/Max_Timeout_Angle ; // Inline DIV
      temp += Fix_Time ;
      CC7 = temp + T0  ;
      CC7IR = 0 ;
      CC7IE = 1 ;
      }
   
   if(timeout_tooth[8] == tooth_count) {

      temp = timeout_angle[8] ; 
      temp = ((unsigned long)time_for_60 * temp)/Max_Timeout_Angle ; // Inline DIV
      temp += Fix_Time ;
      CC8 = temp + T0  ;
      CC8IR = 0 ;
      CC8IE = 1 ;
      }
   
   if(timeout_tooth[9] == tooth_count) {

      temp = timeout_angle[9] ; 
      temp = ((unsigned long)time_for_60 * temp)/Max_Timeout_Angle ; // Inline DIV
      temp += Fix_Time ;
      CC9 = temp + T0  ;
      CC9IR = 0 ;
      CC9IE = 1 ;
      }
   
   if(timeout_tooth[10] == tooth_count) {

      temp = timeout_angle[10] ; 
      temp = ((unsigned long)time_for_60 * temp)/Max_Timeout_Angle ; // Inline DIV
      temp += Fix_Time ;
      CC10 = temp + T0  ;
      CC10IR = 0 ;
      CC10IE = 1 ;
      }
   
   if(timeout_tooth[11] == tooth_count) {

      temp = timeout_angle[11] ; 
      temp = ((unsigned long)time_for_60 * temp)/Max_Timeout_Angle ; // Inline DIV
      temp += Fix_Time ;
      CC11 = temp + T0  ;
      CC11IR = 0 ;
      CC11IE = 1 ;
      }

   /*** Count Teeth ***/

   tooth_count++ ; 
   if(tooth_count == No_Of_Cylinders) {

      tooth_count = 0 ;
      }


   /*** Calculate 16-bit Engine Speed ***/

   y_value = engine_speed = Speed_Div_Const / (unsigned long) time_for_60 ;

   /*** Update Appropriate DP Ram Pressure Channel ***/

   x_value = ADDAT2 & 0x3ff ;  // Load up parameters for interpolation
   map_base = default_map4 ;  // to get inj PW
   map_interp() ;
   master_inj_PW = interp_result ;

   time_last_60 = CC15 ;  // Store current CC15 for next time...
   }


/*** Injector On Interrupts To Determine On-Time (Could be PEC-driven) ***/

void CC0_int(void) interrupt 0x10 {

   CC0 += master_inj_PW ;
   CC0IE = 0 ;      // No more interrupts until next ref
   }

void CC1_int(void) interrupt 0x11 {

   CC1 += master_inj_PW ;
   CC1IE = 0 ;      // No more interrupts until next ref
   }

void CC2_int(void) interrupt 0x12 {

   CC2 += master_inj_PW ;
   CC2IE = 0 ;      // No more interrupts until next ref
   }

void CC3_int(void) interrupt 0x13 {

   CC3 += master_inj_PW ;
   CC3IE = 0 ;      // No more interrupts until next ref
   }

void CC4_int(void) interrupt 0x14 {

   CC4 += master_inj_PW ;
   CC4IE = 0 ;      // No more interrupts until next ref
   }

void CC5_int(void) interrupt 0x15 {

   CC5 += master_inj_PW ;
   CC5IE = 0 ;      // No more interrupts until next ref
   }

void CC6_int(void) interrupt 0x16 {

   CC6 += master_inj_PW ;
   CC6IE = 0 ;      // No more interrupts until next ref
   }

void CC7_int(void) interrupt 0x17 {

   CC7 += master_inj_PW ;
   CC7IE = 0 ;      // No more interrupts until next ref
   }

void CC8_int(void) interrupt 0x18 {

   CC8 += master_inj_PW ;
   CC8IE = 0 ;      // No more interrupts until next ref
   }

void CC9_int(void) interrupt 0x19 {

   CC9 += master_inj_PW ;
   CC9IE = 0 ;      // No more interrupts until next ref
   }

void CC10_int(void) interrupt 0x1a {

   CC10 += master_inj_PW ;
   CC10IE = 0 ;      // No more interrupts until next ref
   }

void CC11_int(void) interrupt 0x1b {

   CC11 += master_inj_PW ;
   CC11IE = 0 ;      // No more interrupts until next ref
   }



/*** Set Up Comms ***/

void v24ini1_166(void) {

  /* initialize the serial interface */

   P3  |= 0x0400;        /* SET PORT 3.10 OUTPUT LATCH (TXD)              */
   DP3 |= 0x0400;        /* SET PORT 3.10 DIRECTION CONTROL (TXD OUTPUT)  */

   S0TIC = 0x80;         /* SET TRANSMIT INTERRUPT FLAG                   */
   S0RIC = 0x00;         /* DELETE RECEIVE INTERRUPT FLAG                 */
   S0BG  = 0x40;         /* SET BAUDRATE TO 9600 BAUD                     */
   S0CON = 0x8011;       /* SET SERIAL MODE                               */
   printf("%c",0x0c) ;   // Clear screen
   }



void init_AD(void) {

   ADCON = 0x0011 ; // Set single channel (1) continuous conversion mode
   ADST = 1 ;  // Start continuous conversion

   ADCIN = 1 ; // Enable channel injection mode
   ADWR = 1 ;  // Enable wait for read mode
   
   ADDAT2 = 0x0000 ; // Set channel 0 for injection mode

   PECC0 = 0x01FF ; // Byte, no count, fixed source and destination addresses
   SRCP0 = (unsigned short)&ADDAT ;                  // Source is ADDAT
   DSTP0 = _sof_(&inj_delay_angle) ;  // Destination is inj angle variable
   
   Set_Priority_14(ADCIC) ;  // Set up AD service via PEC channel 0
   Set_Group_Lvl_0(ADCIC) ; 

   ADCIE = 1 ; // Enable ADC interrupt
   }


/*** Main Loop ***/

void main(void) {

   v24ini1_166() ;
   init_AD() ;
   T0_init() ;

   IEN = 1 ;

   while(1) { 

      drive_terminal() ;
      }
   }
